@@ -13,29 +13,15 @@ Must begin with a pandas DataFrame containing 'wide' data where:
 
 """
 
+from .plotting import plot, AnimatedAccessor, animate_multiple_plots
+from .base import load_dataset
+
+version = "2020.05.15"
 
 # Register animated_plot accessor for Pandas DataFrames and Series:
 import pandas as pd
 from pandas.core.accessor import CachedAccessor
-from .plotting import AnimatedAccessor, plot, animate_multiple_plots
-
-from .base import load_dataset
-
-version = "0.2.4"
-
 
 plot_animated = CachedAccessor("plot_animated", AnimatedAccessor)
 pd.DataFrame.plot_animated = plot_animated
-pd.Series.plot_animated = plot_animated
-
-# Define plot_animated method for GeoPandas and Series:
-
-try:
-    import geopandas as gpd
-    from .geoplotting import geoplot
-
-    gpd.GeoDataFrame.plot_animated = geoplot
-    gpd.GeoSeries.plot_animated = geoplot
-
-except ImportError:
-    pass
+pd.Series.plot_animated = plot
